from blends.models import (
    NId,
)
from blends.query import (
    adj_ast,
)
from blends.syntax.builders.parameter_list import (
    build_parameter_list_node,
)
from blends.syntax.models import (
    SyntaxGraphArgs,
)


def reader(args: SyntaxGraphArgs) -> NId:
    graph = args.ast_graph
    c_ids = (
        _id
        for _id in adj_ast(graph, args.n_id)
        if graph.nodes[_id]["label_type"] not in {"(", ")", ",", "[", "]"}
    )
    return build_parameter_list_node(args, c_ids=c_ids)
