from blends.models import (
    NId,
)
from blends.syntax.models import (
    SyntaxGraphArgs,
)


def build_return_node(args: SyntaxGraphArgs, value_id: NId | None) -> NId:
    args.syntax_graph.add_node(
        args.n_id,
        label_type="Return",
    )
    if value_id:
        args.syntax_graph.update_node_attribute(args.n_id, "value_id", value_id)
        args.syntax_graph.add_edge(
            args.n_id,
            args.generic(args.fork_n_id(value_id)),
            label_ast="AST",
        )

    return args.n_id
